//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This shows how to make your code frame rate
///			independant. If you change the window size the 
///			object will still spin at the same speed.
//------------------------------------------------------------

#include <stdlib.h>
#include <GL/glut.h>
#include "FrameTimer.h"

/// the rotation of the teapot
double g_Rotation=0;

//------------------------------------------------------------	OnIdle()
//
void OnIdle() {

	// update the frame time
	SortFrameTimer();

	//	if our prog runs at 50fps, each frame should take 0.02 secs.
	// 	If we want a value to increase by 10 units in one second,
	//	then each frame we need to increment the value by 0.2 units
	//
	//		Increment Size =  10 units / 50 fps
	//
	// 	Since 1 sec / 50 fps  == FrameTime, we can say
	//
	//		Increment Size = 10 * 1/50 fps
	// 		Increment Size = 10 * FrameTime
	//
	//	In this case, we increment the rotation value by 90
	// 	units per second
	//
	g_Rotation += 90.0f * FrameTime();

	// redraw the screen
	glutPostRedisplay();
}

//------------------------------------------------------------	OnDraw()
//
void OnDraw() {

	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// set the camera position
	gluLookAt(	1,1,10,	//	eye pos
				0,0,0,	//	aim point
				0,1,0);	//	up direction

	// rotate the object
	glRotatef(g_Rotation,0,1,0);

	// draw a simple teapot
	glutWireTeapot(1);

	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {
	// enable depth testing
	glEnable(GL_DEPTH_TEST);
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
}


//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	// prevent division by zero when the window gets minised
	if (h==0)
		h=1;

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	gluPerspective(45.0f,(float)w/h,0.1f,100.0f);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
}


//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	// initialise glut
	glutInit(&argc,argv);

	// request a depth buffer, RGBA display mode, and we want double buffering
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);

	// set the initial window size
	glutInitWindowSize(640,480);

	// create the window
	glutCreateWindow("A basic glut example");

	// set the function to use to draw our scene
	glutDisplayFunc(OnDraw);

	// set the function to handle changes in screen size
	glutReshapeFunc(OnReshape);

	// set the idle callback
	glutIdleFunc(OnIdle);
	
	// run our custom initialisation
	OnInit();

	// set the function to be called when we exit
	atexit(OnExit);

	// initialise the frame timer
	InitFrameTimer();

	// this function runs a while loop to keep the program running.
	glutMainLoop();
	return 0;
}
